package keskustelubotti;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * Emoikkuna, eli pikkuna joka kynnistyy kun botti kynnistetn,
 * sdetn kohdalleen, rakennetaan ja asetetaan tapahtumaksittelijt.
 *
 * Sislt mys tarvittavat rajapintametodit, joilla emoikkunan palveluita kytetn.
 *
 * @author Samuli Kaipiainen
 * @version 0.1
 */

public class Emoikkuna extends JFrame implements BottiUI
{
	// emoikkunan tavarat

	// menupalkki
	private final JMenuBar menu = new JMenuBar();

	// menut menupalkkiin
	private final JMenu menuBotti = new JMenu("Botti");
	private final JMenu menuOhje = new JMenu("Ohje");

	// menuitemit menupalkkiin
	private final JMenuItem menuBottiAsetukset = new JMenuItem("Asetukset");
	private final JMenuItem menuBottiOpettele = new JMenuItem("Opettele");
	private final JMenuItem menuBottiPois = new JMenuItem("Pois");
	private final JMenuItem menuOhjeApua = new JMenuItem("Apua");
	private final JMenuItem menuOhjeTietoja = new JMenuItem("Tietoja");

	// jakaa ikkunan yl- ja alaosaan
	private final JPanel alusta = new JPanel();

	// muu ikkunaan tuleva sl
	private final JTextField tekstirivi = new JTextField();
	private final JTextArea tekstilaatikko = new JTextArea();
	// tekstilaatikko tulee vierityspaneelin sisn
	private final JScrollPane vierityspaneeli = new JScrollPane(tekstilaatikko);
	private final JLabel tilarivi = new JLabel();

	// IIK (hmpptekstej)
	private final String bottiNameVersion = "Keskustelubotti 0.1";
	private final String bottiCopyright = "Samuli Kaipiainen / 2003";

	// ohjeikkuna
	private final Ohjeikkuna ohje;

	// itse emoikkuna tapahtumaksittelijit varten
	private final Emoikkuna emo;

	// botin kyttmt asetukset ja sanavarasto
	private final Asetukset asetukset;
	private final Sanavarasto sanavarasto;

	// bottithread
	private final Botti botti;

	/**
	 * Rakennetaan emoikkuna, listn tapahtumaksittelijt
	 *
	 * @param asetukset Main-luokasta tulevat kaikkien yhteiset asetukset
	 */
	public Emoikkuna(final Asetukset asetukset)
	{
		// rakennetaan menut
		menu.add(menuBotti);
		  menuBotti.add(menuBottiAsetukset);
		  menuBotti.add(menuBottiOpettele);
		  menuBotti.addSeparator();
		  menuBotti.add(menuBottiPois);
		menu.add(menuOhje);
		  menuOhje.add(menuOhjeApua);
		  menuOhje.addSeparator();
		  menuOhje.add(menuOhjeTietoja);

		// alt-pikanppimet menuille
		menuBotti.setMnemonic(KeyEvent.VK_B);
		  menuBottiAsetukset.setMnemonic(KeyEvent.VK_A);
		  menuBottiOpettele.setMnemonic(KeyEvent.VK_O);
		  menuBottiPois.setMnemonic(KeyEvent.VK_P);
		menuOhje.setMnemonic(KeyEvent.VK_O);
		  menuOhjeApua.setMnemonic(KeyEvent.VK_A);
		  menuOhjeTietoja.setMnemonic(KeyEvent.VK_T);

		// muut pikanppimet menuille
		menuBottiOpettele.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, KeyEvent.CTRL_MASK));

		// asetuksia ei ole toteutettu, MUR
		menuBottiAsetukset.setEnabled(false);

		// humppaillaan tekstilaatikko (ja vierityspalkki) kivaksi
		tekstilaatikko.setEditable(false);
		tekstilaatikko.setLineWrap(true);
		tekstilaatikko.setWrapStyleWord(true);
		vierityspaneeli.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);

		// tekstirivist ei pse pois tabulaattorilla (vaatii javan version 1.4; muu botti prjnnee 1.3:lla)
		tekstirivi.setFocusTraversalKeysEnabled(false);

		// alustetaan alusta, joka tulee ikkunan ylosaan jakaen ikkunan kahteen osaan
		alusta.setLayout(new BorderLayout());

		// listn tavara alustaan
		alusta.add(vierityspaneeli, BorderLayout.CENTER);
		alusta.add(tekstirivi, BorderLayout.SOUTH);

		// listn tavara emoikkunaan
		this.setJMenuBar(menu);
		this.getContentPane().add(alusta, BorderLayout.CENTER);
		this.getContentPane().add(tilarivi, BorderLayout.SOUTH);

		// rakennetaan ohjeikkuna valmiiksi
		ohje = new Ohjeikkuna(asetukset.ohjetiedosto);

		// emoikkuna tapahtumaksittelijiden nimettmi aliluokkia varten
		emo = this;

		// tapahtumaksittelij tekstiriville
		tekstirivi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e)
			{
				// jos bottia ei ole viel luotu (esim. luetaan isoa sanatiedostoa), ei tehd mitn
				if (botti == null) {
					tekstirivi.selectAll();
					return;
				}

				String teksti = tekstirivi.getText();

				if (teksti.length() > 0) {
					sano(asetukset.nick, teksti);

					tekstirivi.setText("");

					botti.uusiLause(teksti);
				}
			}
		});
/*
		// huoh, jlkeenpinen ajatus kaivella vanhoja tekstej painamalla yls/alas;
		// dumpataan ja arkistoidaan, kun ei jaksa
		tekstirivi.addKeyListener(new KeyAdapter() {
			public void keyPressed(KeyEvent e) {
				// Main.debug(String.valueOf(e.getKeyCode()));
				// if (e.getKeyCode() == KeyEvent.VK_UP) tilarivi.setText("yls");
				// if (e.getKeyCode() == KeyEvent.VK_DOWN) tilarivi.setText("alas");
			}
		});
*/
		// tapahtumaksittelijt menuitemeille

		menuBottiAsetukset.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e)
			{
			}
		});

		menuBottiOpettele.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e)
			{
				JFileChooser tv = new JFileChooser();

				tv.setDialogTitle("Valitse opeteltava tekstitiedosto");
				tv.setApproveButtonToolTipText("Opettele valittu tiedosto");
				int nappi = tv.showDialog(emo, "Opettele");

				if (nappi == JFileChooser.APPROVE_OPTION) {
					String tiedosto = tv.getSelectedFile().getPath();
					ajattele("opetellaan <- '" + tiedosto + "'");
					// laitetaan botti opettelemaan tiedostoa
					if (!botti.opettele(tiedosto))
						ajattele("virhe avattaessa <- '" + tiedosto + "'");
				}
			}
		});

		menuBottiPois.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e)
			{
				pois();
			}
		});

		menuOhjeApua.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e)
			{
				ohje.show();
			}
		});

		menuOhjeTietoja.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e)
			{
				JOptionPane.showMessageDialog(emo,
				    bottiNameVersion + " / lski\n\n" + bottiCopyright,
				    bottiNameVersion, JOptionPane.INFORMATION_MESSAGE,
				    new ImageIcon(asetukset.bottiKuvatiedosto));
			}
		});

		// tapahtumaksittelij ikkunan sulkemiselle
		this.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e)
			{
			       pois();
			}
		});

		// sdetn ikkunaa, asetetaan nkyvksi ja ollaan iloisia
		this.setTitle(bottiNameVersion + " *murh*");
		this.setSize(600, 400);
		this.show();
		// kursori valmiiksi vilkkumaan tekstiriviin, ilman ett tytyy sohia hiirell aluksi
		tekstirivi.requestFocusInWindow();

		// Mainilta saadut asetukset
		this.asetukset = asetukset;

		// luodaan uusi sanavarasto
		this.sanavarasto = new Sanavarasto();

		// luetaan sanavarasto sanatiedostosta
		ajattele("luetaan sanavarastoa <- " + asetukset.sanatiedosto);
		if (sanavarasto.lueTiedostosta(asetukset.sanatiedosto))
			ajattele("sanavarasto luettu (" + asetukset.sanatiedosto + ", " +
				 sanavarasto.sanoja() + " sanaa)");
		else
			ajattele("virhe luettaessa sanavarastoa " + asetukset.sanatiedosto +
				 ", luotiin tyhj sanavarasto");

		// luodaan uusi botti juttelukaveriksi
		botti = new Botti(this, asetukset, sanavarasto);
	}

	/**
	 * Emoikkunan rajapinta, tmn avulla "sanotaan" kaikki emoikkunaan tuleva teksti,
	 * Botti kutsuu emoikkunan sano-metodia kun haluaa sanoa jotain.
	 *
	 * ...
	 * invokeLater-sekoilu tarvitaan, jos halutaan ett tekstilaatikko scrollaa mys sen jlkeen,
	 * kun botti on sanonut jotain (botti-thread kutsunut sano-metodia).
	 *
	 * Vaikka JTextArea:n append-metodi onkin thread-safe, jostain syyst muualta kuin event-dispatching
	 * threadista kutsutun appendin jlkeen JTextArea ei en scrollaa automaattisesti alaspin teksti
	 * listtess. Niinp tytyy kytt javax.swing.SwingUtilities.invokeLater-metodia, joka taas tahtoo
	 * wrapperiluokan jonka run-metodia kutsuu kun swing on valmis pivittelemn palikoitaan meidn takia.
	 *
	 * ...Herra tiet mit kauheuksia tapahtuisi, jos append ei olisi edes thread-safe, ja sit yrittisi
	 * ronkkia suoraan jostakin puun juuresta putkahtaneesta threadista!
	 *
	 * @param kuka Kuka sanoo
	 * @param mit Mit sanotaan
	 */
	public void sano(String kuka, String mit)
	{
		final StringBuffer teksti = new StringBuffer();

		// listn rivinvaihto, paitsi jos tekstilaatikko on tyhj
		if (tekstilaatikko.getLineCount() > 1 ||
		    tekstilaatikko.getText().length() > 0) teksti.append("\n");

		// kellonaika
		teksti.append("[" + Main.aikaleima() + "] ");

		// nimitagi
		teksti.append("<" + kuka + "> ");

		// itse teksti
		teksti.append(mit);

		// listn uusi teksti tekstilaatikkoon
		// (ei scrollaa alaspin sen jlkeen kun botti-thread on kynyt tll)
		tekstilaatikko.append(teksti.toString());

		// humpataan tekstilaatikko scrollaamaan
		javax.swing.SwingUtilities.invokeLater(new Runnable() {
			public void run()
			{
				// rullataan tekstilaatikon loppuun
				tekstilaatikko.scrollRectToVisible(new Rectangle(0, Integer.MAX_VALUE, 0, 0));

				// voisi mys appendata vasta invokelaterissa, mutta tekstilaatikkoa
				// floodatessa saattaa osa riveist jd nyttmtt (ehk, en tied),
				// ja lisksi kyttliittym jumittaa nin tehtess
			}
		});

	}

	/**
	 * Emoikkunan rajapinta, tmn avulla botti kertoo mahdollisia (lhinn debugmaisia)
	 * ajatuksiaan. Jotain turhia, murmur MUR, murh. Ei kai kovin paljoa kuitenkaan.
	 *
	 * Mys emoikkuna pohtii ajattele-metodin avulla.
	 *
	 * @param mit Mit tll kertaa ajattellaan
	 */
	public void ajattele(String mit)
	{
		String pohdinta = "[" + Main.aikaleima() + "] *" + mit + "*";

		tilarivi.setText(pohdinta);

		// infotaan mys konsoliin (jos debugKonsoli-tila)
		Main.debugKonsoli(pohdinta);
	}

	/**
	 * Tapetaan botti, ensin tallennetaan opitut sanat sanatiedostoon
	 */
	private void pois()
	{
		ajattele("tallennetaan sanavarasto -> " + asetukset.sanatiedosto);
		// pivtetn ikkuna, jotta teksti tulee nkyviin
		paint(getGraphics());
		// (tai) tilarivi.paintImmediately(tilarivi.getVisibleRect());
		sanavarasto.tallennaTiedostoon(asetukset.sanatiedosto);

		System.exit(0);
	}
}
