package keskustelubotti;

/**
 * Generoi satunnaisia lauseita botin sanavarastosta.
 *
 * @author Samuli Kaipiainen
 * @version 0.1
 */

public class Lausegeneroija
{
	private final Sanavarasto sanavarasto;
	private final Asetukset asetukset;

	/**
	 * Luo uuden lausegeneroijan
	 *
	 * @param sanavarasto Mit sanavarastoa generoija kytt
	 * @param asetukset Kytettvt asetukset
	 */
	public Lausegeneroija(Sanavarasto sanavarasto, Asetukset asetukset)
	{
		this.sanavarasto = sanavarasto;
		this.asetukset = asetukset;
	}

	/**
	 * Generoi uuden satunnaisen lauseen, jossa vhintn yksi sana;
	 * botti sanoo mit sattuu (ja yleens mys sattuu)
	 *
	 * @param sanoja Kuinka monta sanaa lauseeseen tungetaan
	 * @return Generoitu lause; null jos sanavarasto on tyhj
	 */
	public String satunnainenLause(int sanoja)
	{
		if (sanavarasto.onTyhj()) return null;

		StringBuffer lause = new StringBuffer();
		String ekaSana = sanavarasto.annaSatunnainenSana();

		// ensimminen sana isolla alkukirjaimella
		lause.append(ekaSana.substring(0, 1).toUpperCase());
		lause.append(ekaSana.substring(1));

		for (int n = 1; n < sanoja; n++) {
			// pilkutetaan aina vlill, paitsi ei viimeist sanaa
			if (Main.random() < asetukset.pilkkuTN) lause.append(",");
			lause.append(" ");
			lause.append(sanavarasto.annaSatunnainenSana());
		}

		// loppuun piste, huuto- tai kysymysmerkki
		double rnd = Main.random();
		if (rnd < asetukset.kysymysTN) lause.append("?");
		else if (rnd < asetukset.kysymysTN + asetukset.huutoTN) lause.append("!");
		else lause.append(".");

		// listn kirjoitusvirheet
		typota(lause);

		return lause.toString();
	}

	/**
	 * Lis annettuun lauseeseen kirjoitusvirheit, murh, hlmn raskas operaatio
	 *
	 * @param lause Typotettava lause
	 */
	private void typota(StringBuffer lause)
	{
		for (int n = 0; n < lause.length(); n++) {
			double rnd = Main.random(), summaTN = 0;

			if (rnd < (summaTN += asetukset.typoPoistoTN)) {
				// poistetaan merkki
				lause.deleteCharAt(n--);

			} else if (rnd < (summaTN += asetukset.typoVaihtoTN)) {
				// vaihdetaan edellisen kanssa, paitsi jos ollaan ensimmisen merkin kohdalla
				if (n > 0) {
					char temp = lause.charAt(n);
					lause.setCharAt(n, lause.charAt(n - 1));
					lause.setCharAt(n - 1, temp);
				}

			} else if (rnd < (summaTN += asetukset.typoMuuttoTN)) {
				// muutetaan satunnaiseksi merkiksi
				lause.setCharAt(n, (char) (' ' + Main.random() * ('~' - ' ')));

			} else if (rnd < (summaTN += asetukset.typoLisysTN)) {
				// listn satunnainen merkki
				lause.insert(n++, (char) (' ' + Main.random() * ('~' - ' ')));
			}
		}
	}
}
