package keskustelubotti;

import java.util.Calendar;
import java.util.Random;

/**
 * Pieni, turha mutta helposti lydettv alkuluokka, josta luodaan emoikkuna.
 *
 * Sislt lisksi viitteen ohjelman asetuksiin, lhinn debug-tilan tunnistamista varten,
 * sek joitakin ktevi ja luonteensa vuoksi tnne sopivia 'kirjastorutiineja',
 * kuten debug, joka tulostaa konsoliin turhaa tietoa niin haluttaessa,
 * ja Random-wrapperit, joita muut luokat kutsuvat.
 *
 * @author Samuli Kaipiainen
 * @version 0.1
 */

public final class Main
{
	// meidn yhteiset asetuksemme, emoikkuna aivan suotta, sek satunnaisgeneraattori
	private static Asetukset asetukset;
	private static Emoikkuna emo;
	private static Random random;

	// ei ilmentymi, plh (eik aliluokkia, siksi luokka on final, Plh) *hajoilee ykoodaamiseen*
	private Main() { }

	/**
	 * luodaan emoikkuna, asetukset ja satunnaisgeneraattori
	 *
	 * @param args Komentoriviparametrit
	 */
	public static void main(String[] args)
	{
		random = new Random(System.currentTimeMillis());
		asetukset = new Asetukset();
		emo = new Emoikkuna(asetukset);
	}

	/**
	 * Tulostaa annetun merkkijonon emoikkunan tilariville, jos debugTilarivi-tila on
	 * asetuksissa pll, tai jos ei ole mutta debugKonsoli-tila on pll,
	 * tulostaa merkkijonon konsoliin (System.out.println)
	 *
	 * Siksi sotkuista purkkaa, kun emoikkunan ajattele-metodi kutsuu debugKonsoli-metodia
	 *
	 * @param s Mit tulostetaan
	 */
	public static void debug(String s)
	{
		if (asetukset.debugTilarivi) emo.ajattele(s);
		else if (asetukset.debugKonsoli) System.out.println(s);
	}

	/**
	 * Tulostaa annetun merkkijonon konsoliin (System.out.println),
	 * mikli debugKonsoli-tila on asetuksissa pll, mutta ei
	 * tulosta emoikkunan tilariville, vaikka debugTilarivi-tila olisikin pll
	 *
	 * Emoikkunan ajattele-metodi kutsuu tt, koska se lis tekstin itse tilariville
	 *
	 * @param s Mit tulostetaan
	 */
	public static void debugKonsoli(String s)
	{
		if (asetukset.debugKonsoli) System.out.println(s);
	}

	/**
	 * Virheilmoitusten dumppauspaikka, tulostaa annetun merkkijonon
	 * virhekonsoliin (System.err.println)
	 *
	 * @param s Mit tulostetaan
	 */
	public static void error(String s)
	{
		System.err.println(s);
	}

	/**
	 * @return Tmnhetkinen kellonaika muodossa hh:mm
	 */
	public static String aikaleima()
	{
		Calendar aika = Calendar.getInstance();
		StringBuffer ret = new StringBuffer();

		int tunti = aika.get(Calendar.HOUR_OF_DAY);
		int minuutti = aika.get(Calendar.MINUTE);

		if (tunti < 10) ret.append("0");
		ret.append(tunti);

		ret.append(":");

		if (minuutti < 10) ret.append("0");
		ret.append(minuutti);

		return ret.toString();
	}

	/**
	 * @return Satunnainen double [0,1[
	 */
	public static double random()
	{
		return random.nextDouble();
	}

	/**
	 * @param pienin Pienin arvottava luku
	 * @param ylraja Suurin+1 arvottava luku
	 * @return Satunnainen int [pienin,ylraja[
	 */
	public static int randomint(int pienin, int ylraja)
	{
		return pienin + random.nextInt(ylraja - pienin);
	}

	/**
	 * @return Satunnainen normaalijakauman (KA 0, KH 1) mukainen double
	 */
	public static double randomGauss()
	{
		return random.nextGaussian();
	}

	/**
	 * @param ka Keskiarvo
	 * @param kh Keskihajonta
	 * @return Satunnainen normaalijakauman (ka, kh) mukainen double
	 */
	public static int randomGauss(int ka, int kh)
	{
		return (int) (ka + randomGauss() * kh);
	}
}
