package keskustelubotti;

import java.io.*;

/**
 * Kirjoittaa tiedostoon rivi kerrallaan.
 *
 * @author Samuli Kaipiainen
 * @version 0.1
 */

public class Tiedostokirjoitin
{
	private final String tiedosto;
	// ei voi olla final, koska java itkee ett se joko saa arvon tai ei saa konstruktorin try-lohkossa
	private BufferedWriter out;
	private boolean avattuOK;

	/**
	 * Luo uuden tiedostokirjoittimen, jos tiedoston avaus onnitui, avattuOK palauttaa true
	 *
	 * @param tiedosto Mihin kirjoitetaan
	 */
	public Tiedostokirjoitin(String tiedosto)
	{
		this.tiedosto = tiedosto;
		avattuOK = true;

		try {
			out = new BufferedWriter(new FileWriter(tiedosto));
		} catch (IOException e) {
			Main.error("virhe avattaessa '" + tiedosto + "': " + e.getMessage());
			avattuOK = false;
		}
	}

	/**
	 * @return true jos tiedoston avaus konstruktorissa onnistui; muuten false
	 */
	public boolean avattuOK()
	{
		return avattuOK;
	}

	/**
	 * Sulkee avatun tiedoston, jonka jlkeen tll ilmentymll ei tee en mitn
	 *
	 * @return true jos sulkeminen onnistui; muuten false
	 */
	public boolean sulje()
	{
		if (!avattuOK) return false;

		try {
			out.close();
		} catch (IOException e) {
			Main.error("virhe suljettaessa '" + tiedosto + "': " + e.getMessage());
			return false;
		}

		return true;
	}

	/**
	 * Kirjoittaa seuraavan rivin tiedostoon
	 *
	 * @param s Kirjoitettava rivi
	 * @return true jos kirjoitus onnistui; muuten false
	 */
	public boolean seuraavaRivi(String s)
	{
		if (!avattuOK) return false;

		try {
			out.write(s);
			out.newLine();
		} catch (IOException e) {
			Main.error("virhe kirjoitettaessa ->'" + tiedosto + "': " + e.getMessage());
			return false;
		}

		return true;
	}
}
